@@ -12,6 +12,7 @@
 
 ****************************************************************************/
 
+#include "SetOpTemplate.h"
 #define BUCKETTEMPLATE_C "$Id$\n"
 
 /* Use BUCKET_SEARCH to find the index at which a key belongs.
@@ -1299,8 +1300,7 @@ _bucket_setstate(Bucket *self, PyObject *state)
     }
 
     len = PyTuple_Size(items);
-    if (len < 0)
-        return -1;
+    ASSERT(len >= 0, "_bucket_setstate: items tuple has negative size", -1);
     len /= 2;
 
     for (i = self->len; --i >= 0; ) {
@@ -1367,6 +1367,26 @@ bucket_setstate(Bucket *self, PyObject *state)
     return Py_None;
 }
 
+static PyObject *
+bucket_sub(PyObject *self, PyObject *other)
+{
+    PyObject *args = Py_BuildValue("OO", self, other);
+    return difference_m(NULL, args);
+}
+
+static PyObject *
+bucket_or(PyObject *self, PyObject *other)
+{
+    PyObject *args = Py_BuildValue("OO", self, other);
+    return union_m(NULL, args);
+}
+
+static PyObject *
+bucket_and(PyObject *self, PyObject *other)
+{
+    PyObject *args = Py_BuildValue("OO", self, other);
+    return intersection_m(NULL, args);
+}
 
 static PyObject *
 bucket_setdefault(Bucket *self, PyObject *args)
@@ -1386,7 +1406,7 @@ bucket_setdefault(Bucket *self, PyObject *args)
     /* The key isn't in the bucket.  If that's not due to a KeyError exception,
      * pass back the unexpected exception.
      */
-    if (! PyErr_ExceptionMatches(PyExc_KeyError))
+    if (! BTree_ShouldSuppressKeyError())
         return NULL;
     PyErr_Clear();
 
@@ -1427,7 +1447,7 @@ bucket_pop(Bucket *self, PyObject *args)
     /* The key isn't in the bucket.  If that's not due to a KeyError exception,
      * pass back the unexpected exception.
      */
-    if (! PyErr_ExceptionMatches(PyExc_KeyError))
+    if (! BTree_ShouldSuppressKeyError())
         return NULL;
 
     if (failobj != NULL) {
@@ -1445,6 +1465,41 @@ bucket_pop(Bucket *self, PyObject *args)
     return NULL;
 }
 
+static PyObject*
+bucket_popitem(Bucket* self, PyObject* args)
+{
+    PyObject* key = NULL;
+    PyObject* pop_args = NULL;
+    PyObject* result_val = NULL;
+    PyObject* result = NULL;
+
+    if (PyTuple_Size(args) != 0) {
+        PyErr_SetString(PyExc_TypeError, "popitem(): Takes no arguments.");
+        return NULL;
+    }
+
+    key = Bucket_minKey(self, args); /* reuse existing empty tuple. */
+    if (!key) {
+        PyErr_Clear();
+        PyErr_SetString(PyExc_KeyError, "popitem(): empty bucket.");
+        return NULL;
+    }
+
+    pop_args = PyTuple_Pack(1, key);
+    if (pop_args) {
+        result_val = bucket_pop(self, pop_args);
+        Py_DECREF(pop_args);
+        if (result_val) {
+            result = PyTuple_Pack(2, key, result_val);
+            Py_DECREF(result_val);
+        }
+    }
+
+    Py_DECREF(key);
+    return result;
+}
+
+
 /* Search bucket self for key.  This is the sq_contains slot of the
  * PySequenceMethods.
  *
@@ -1463,7 +1518,7 @@ bucket_contains(Bucket *self, PyObject *key)
         result = INT_AS_LONG(asobj) ? 1 : 0;
         Py_DECREF(asobj);
     }
-    else if (PyErr_ExceptionMatches(PyExc_KeyError)) {
+    else if (BTree_ShouldSuppressKeyError()) {
         PyErr_Clear();
         result = 0;
     }
@@ -1502,7 +1557,7 @@ bucket_getm(Bucket *self, PyObject *args)
         PyErr_Clear();
         PyErr_SetObject(PyExc_KeyError, key);
     }
-    if (!PyErr_ExceptionMatches(PyExc_KeyError))
+    if (!BTree_ShouldSuppressKeyError())
         return NULL;
     PyErr_Clear();
     Py_INCREF(d);
@@ -1701,6 +1756,10 @@ static struct PyMethodDef Bucket_methods[] = {
      "If key is not found, d is returned if given, otherwise KeyError\n"
      "is raised."},
 
+    {"popitem", (PyCFunction)bucket_popitem, METH_VARARGS,
+     "D.popitem() -> (k, v), remove and return some (key, value) pair\n"
+     "as a 2-tuple; but raise KeyError if D is empty."},
+
     {"iterkeys", (PyCFunction) Bucket_iterkeys, METH_VARARGS | METH_KEYWORDS,
      "B.iterkeys([min[,max]]) -> an iterator over the keys of B"},
 
@@ -1847,6 +1906,29 @@ static PySequenceMethods Bucket_as_sequence = {
     0,                                  /* sq_inplace_repeat */
 };
 
+static PyNumberMethods Bucket_as_number = {
+     (binaryfunc)0,                     /* nb_add */
+     bucket_sub,                        /* nb_subtract */
+     (binaryfunc)0,                     /* nb_multiply */
+#ifndef PY3K
+     0,                                 /* nb_divide */
+#endif
+     (binaryfunc)0,                     /* nb_remainder */
+     (binaryfunc)0,                     /* nb_divmod */
+     (ternaryfunc)0,                    /* nb_power */
+     (unaryfunc)0,                      /* nb_negative */
+     (unaryfunc)0,                      /* nb_positive */
+     (unaryfunc)0,                      /* nb_absolute */
+     (inquiry)0,                        /* nb_bool */
+     (unaryfunc)0,                      /* nb_invert */
+     (binaryfunc)0,                     /* nb_lshift */
+     (binaryfunc)0,                     /* nb_rshift */
+     bucket_and,                        /* nb_and */
+     (binaryfunc)0,                     /* nb_xor */
+     bucket_or,                         /* nb_or */
+};
+
+
 static PyObject *
 bucket_repr(Bucket *self)
 {
@@ -1911,7 +1993,7 @@ static PyTypeObject BucketType = {
     0,                                      /* tp_setattr */
     0,                                      /* tp_compare */
     (reprfunc)bucket_repr,                  /* tp_repr */
-    0,                                      /* tp_as_number */
+    &Bucket_as_number,                      /* tp_as_number */
     &Bucket_as_sequence,                    /* tp_as_sequence */
     &Bucket_as_mapping,                     /* tp_as_mapping */
     0,                                      /* tp_hash */
@@ -1920,6 +2002,9 @@ static PyTypeObject BucketType = {
     0,                                      /* tp_getattro */
     0,                                      /* tp_setattro */
     0,                                      /* tp_as_buffer */
+#ifndef PY3K
+    Py_TPFLAGS_CHECKTYPES |
+#endif
     Py_TPFLAGS_DEFAULT |
     Py_TPFLAGS_HAVE_GC |
     Py_TPFLAGS_BASETYPE,                    /* tp_flags */
