@@ -26,8 +26,8 @@
 
 #define MOD_NAME_PREFIX "II"
 
-#define DEFAULT_MAX_BUCKET_SIZE 120
-#define DEFAULT_MAX_BTREE_SIZE 500
+
+
 
 #include "_compat.h"
 #include "intkeymacros.h"
